package com.qmth.teachcloud.common.util;

import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import org.apache.commons.io.IOUtils;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.pdmodel.PDPageTree;
import org.apache.pdfbox.rendering.PDFRenderer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletResponse;
import java.awt.image.BufferedImage;
import java.io.*;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @Description: 转换工具类
 * @Author: CaoZixuan
 * @Date: 2021-04-14
 */

@Component
public class ConvertUtil {
    private final static Logger log = LoggerFactory.getLogger(ConvertUtil.class);

    @Resource
    RedisTemplate<String, Object> redisTemplate;

    // inputStream转outputStream
    public static ByteArrayOutputStream parse(final InputStream in) throws Exception {
        final ByteArrayOutputStream swapStream = new ByteArrayOutputStream();
//        int ch;
//        while ((ch = in.read()) != -1) {
//            swapStream.write(ch);
//        }
        IOUtils.copy(in, swapStream);
        return swapStream;
    }

    // outputStream转inputStream
    public static ByteArrayInputStream parse(final OutputStream out) {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        baos = (ByteArrayOutputStream) out;
        return new ByteArrayInputStream(baos.toByteArray());
    }

    /**
     * 转换，解析时间
     *
     * @param date 日期(考试日期，必须是同一天     example: 2021-04-13)
     * @param time 时间(考试时间，必须使用‘-’隔开  example: 18:00-20:00)
     * @return 开始时间和结束时间键值对(" startTime " : 1618308000000, " endTime " : 1618315200000)
     */
    public static Map<String, Object> analyzeStartAndEndTime(String date, String time) {
        time = time.replaceAll(" ", ""); // 去掉所有空格
        String[] arr = time.split(SystemConstant.HYPHEN);
        if (arr.length != 2) {
            // 不能使用'-'拆分成两个时间的报错
            throw ExceptionResultEnum.ERROR.exception("提供的时间格式异常，考试时间[" + time + "]应该使用如下的格式（考试时间:18:00-20:00）");
        }
        String startTimeStr = date + " " + arr[0];
        String endTimeStr = date + " " + arr[1];
        if (Objects.isNull(DateDisposeUtils.parseDate(startTimeStr)) || Objects.isNull(DateDisposeUtils.parseDate(endTimeStr))) {
            throw ExceptionResultEnum.ERROR.exception("提供的日期时间格式异常 \n考试日期[" + date + "]应该使用如下的格式（考试日期:2021-09-01 或 2021/09/01 或 2021.09.01）" + "\n考试时间[" + time + "]应该使用如下的格式（考试时间:18:00-20:00）");
        }
        long startTime = DateDisposeUtils.parseDate(startTimeStr).getTime();
        long endTime = DateDisposeUtils.parseDate(endTimeStr).getTime();
        if (startTime >= endTime) {
            throw ExceptionResultEnum.ERROR.exception("开始时间要小于结束时间 时间： " + time);
        }

        Map<String, Object> timeMap = new HashMap<>();
        timeMap.put("startTime", startTime);
        timeMap.put("endTime", endTime);
        return timeMap;
    }

    /**
     * 根据开始时间和结束时间解析日期和时间
     *
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @return 键值对
     */
    public static Map<String, Object> analyzeDateAndTime(Long startTime, Long endTime) {
        String tmpDateStart;
        String tmpDateEnd;
        String tmpTimeStart;
        String tmpTimeEnd;
        String date;
        String time;


        Map<String, Object> map = new HashMap<>();

        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date(startTime));
        tmpDateStart = (new SimpleDateFormat("yyyy-MM-dd")).format(calendar.getTime());
        tmpTimeStart = (new SimpleDateFormat("HH:mm:ss")).format(calendar.getTime());

        calendar.setTime(new Date(endTime));
        tmpDateEnd = (new SimpleDateFormat("yyyy-MM-dd")).format(calendar.getTime());
        tmpTimeEnd = (new SimpleDateFormat("HH:mm:ss")).format(calendar.getTime());

        if (!tmpDateStart.equals(tmpDateEnd)) {
            throw ExceptionResultEnum.ERROR.exception("开始时间和结束时间不在同一天");
        }

        date = tmpDateStart;
        time = tmpTimeStart + " - " + tmpTimeEnd;
        map.put("date", date);
        map.put("time", time);

        return map;
    }


    /**
     * 根据开始时间和结束时间解析日期和时间
     *
     * @param startTime 开始时间
     * @param endTime   结束时间
     */
    public static void analyzeAndCompareDateAndTime(Long startTime, Long endTime) {
        String tmpDateStart;
        String tmpDateEnd;

        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date(startTime));
        tmpDateStart = (new SimpleDateFormat("yyyy-MM-dd")).format(calendar.getTime());

        calendar.setTime(new Date(endTime));
        tmpDateEnd = (new SimpleDateFormat("yyyy-MM-dd")).format(calendar.getTime());

        if (!tmpDateStart.equals(tmpDateEnd)) {
            throw ExceptionResultEnum.ERROR.exception("开始时间和结束时间不在同一天");
        }

        if (startTime >= endTime) {
            throw ExceptionResultEnum.ERROR.exception("开始时间必须小于结束时间");
        }
    }

    public static void delFolder(String folderPath) {
        try {
            delAllFile(folderPath); //删除完里面所有内容
            String filePath = folderPath;
            filePath = filePath.toString();
            File myFilePath = new File(filePath);
            myFilePath.delete(); //删除空文件夹
        } catch (Exception e) {
            log.error(SystemConstant.LOG_ERROR, e);
        }
    }

    public static void delAllFile(String path) {
        boolean flag = false;
        File file = new File(path);
        if (!file.exists()) {
            return;
        }
        if (!file.isDirectory()) {
            return;
        }
        String[] tempList = file.list();
        File temp = null;
        for (int i = 0; i < Objects.requireNonNull(tempList).length; i++) {
            if (path.endsWith(File.separator)) {
                temp = new File(path + tempList[i]);
            } else {
                temp = new File(path + File.separator + tempList[i]);
            }
            if (temp.isFile()) {
                temp.delete();
            }
            if (temp.isDirectory()) {
                delAllFile(path + SystemConstant.ORG_SPLIT + tempList[i]);//先删除文件夹里面的文件
                delFolder(path + SystemConstant.ORG_SPLIT + tempList[i]);//再删除空文件夹
                flag = true;
            }
        }
    }

    public static void outputFile(HttpServletResponse response, InputStream inputStream, String fileName) {
        try {
//            byte[] buf = new byte[1024];
//            int len = 0;

            String fName = new String(fileName.getBytes(), "ISO-8859-1");

            response.reset();
            response.setContentType("application/x-msdownload");
            response.setHeader("Content-Disposition", "attachment; filename=" + fName);

//            OutputStream outStream = response.getOutputStream();

//            while ((len = inputStream.read(buf)) > 0) {
//                outStream.write(buf, 0, len);
//            }

            IOUtils.copy(inputStream, response.getOutputStream());
            inputStream.close();
//            outStream.close();
        } catch (IOException e) {
            log.error(SystemConstant.LOG_ERROR, e);
        }
    }

    public static void outputFile(HttpServletResponse response, ByteArrayOutputStream baos, String fileName) throws Exception {
        try {
            String fName = URLEncoder.encode(fileName, SystemConstant.CHARSET_NAME);

            response.reset();
            response.setContentType("application/x-msdownload");
            response.setHeader("Content-Disposition", "attachment; filename=" + fName);

//            OutputStream outStream = response.getOutputStream();
//            baos.writeTo(outStream);
//            outStream.close();
            IOUtils.write(baos.toByteArray(), response.getOutputStream());
        } catch (IOException e) {
            throw new Exception("下载失败");
        }
    }

    public static List<File> pdfToImageFile(String sourcePath, String targetPath) throws Exception {
        PDDocument doc = null;
        ByteArrayOutputStream os = null;
        InputStream stream = null;
//        OutputStream out = null;
        List<File> fileList = new ArrayList<>();
        try {
            // pdf路径
            stream = new FileInputStream(sourcePath);
            // 加载解析PDF文件
            doc = PDDocument.load(stream);
            PDFRenderer pdfRenderer = new PDFRenderer(doc);
            PDPageTree pages = doc.getPages();
            int pageCount = pages.getCount();
            for (int i = 0; i < pageCount; i++) {
                BufferedImage bim = pdfRenderer.renderImageWithDPI(i, 200);
                os = new ByteArrayOutputStream();
                ImageIO.write(bim, "jpg", os);
                // jpg文件转出路径
                File file = new File(targetPath + SystemConstant.HYPHEN + (i + 1) + SystemConstant.JPG_PREFIX);
                if (!file.getParentFile().exists()) {
                    // 不存在则创建父目录及子文件
                    file.getParentFile().mkdirs();
                    file.createNewFile();
                }
//                out = new FileOutputStream(file);
                IOUtils.write(os.toByteArray(), new FileOutputStream(file));
//                out.write(dataList);
                fileList.add(file);
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw e;
        } finally {
            if (doc != null) {
                doc.close();
            }
            if (os != null) {
                os.close();
            }
            if (stream != null) {
                stream.close();
            }
        }
        return fileList;
    }

    public static List<File> pdfToImageFile(File pdfFile) throws Exception {
        PDDocument doc = null;
        ByteArrayOutputStream os = null;
        InputStream stream = null;
        List<File> fileList = new ArrayList<>();
        try {
            // pdf路径
            stream = new FileInputStream(pdfFile);
            // 加载解析PDF文件
            doc = PDDocument.load(stream);
            PDFRenderer pdfRenderer = new PDFRenderer(doc);
            PDPageTree pages = doc.getPages();
            int pageCount = pages.getCount();
            for (int i = 0; i < pageCount; i++) {
                BufferedImage bim = pdfRenderer.renderImageWithDPI(i, 200);
                os = new ByteArrayOutputStream();
                ImageIO.write(bim, "jpg", os);
                // jpg文件转出路径
                File file = SystemConstant.getFileTempVar(SystemConstant.JPG_PREFIX);
                if (!file.getParentFile().exists()) {
                    // 不存在则创建父目录及子文件
                    file.getParentFile().mkdirs();
                    file.createNewFile();
                }
                IOUtils.write(os.toByteArray(), new FileOutputStream(file));
                fileList.add(file);
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw e;
        } finally {
            if (doc != null) {
                doc.close();
            }
            if (os != null) {
                os.close();
            }
            if (stream != null) {
                stream.close();
            }
        }
        return fileList;
    }

}
