package com.qmth.teachcloud.mark.bean.answercard;

import java.awt.Image;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.text.DecimalFormat;
import java.util.Base64;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import javax.imageio.ImageIO;

import com.alibaba.fastjson.JSON;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.qmth.boot.core.exception.ParameterException;
import com.qmth.teachcloud.common.bean.dto.mark.PictureConfig;
import org.apache.commons.lang3.StringUtils;

/**
 * 卡格式文件内容结构
 */
@JsonIgnoreProperties(ignoreUnknown = true)
public class CardFile {

    private String version;

    private boolean adapted;

    private List<CardPageWrapper> pages = Collections.emptyList();

    @JsonIgnore
    private SliceConfig sliceConfig;

    @JsonIgnore
    private MarkConfig markConfig;

    @JsonIgnore
    private List<String> sliceName;

    public String getVersion() {
        return version;
    }

    public void setVersion(String version) {
        this.version = version;
    }

    public boolean isAdapted() {
        return adapted;
    }

    public void setAdapted(boolean adapted) {
        this.adapted = adapted;
    }

    public List<CardPageWrapper> getPages() {
        return pages;
    }

    public void setPages(List<CardPageWrapper> pages) {
        this.pages = pages;
    }

    public static String getMarkConfigAdd(List<PictureConfig> list, double position) {
        for (PictureConfig configItem : list) {
            configItem.setX(configItem.getX() < position ? configItem.getX() : configItem.getX() - position);
            configItem.setY(configItem.getY() < position ? configItem.getY() : configItem.getY() - position);
            configItem.setW(configItem.getW() + position * 2);
            configItem.setH(configItem.getH() + position * 2);
        }
        return JSON.toJSONString(list);
    }

    /**
     * 解析卡格式文件中的裁切图坐标，用于云阅卷同步
     *
     * @return
     */
    public SliceConfig getSliceConfig() throws IOException {
        if (sliceConfig == null) {
            sliceConfig = new SliceConfig();
            if (pages != null) {
                int pageNumber = 0;
                for (CardPageWrapper pageWrapper : pages) {
                    pageNumber++;
                    CardPage page = pageWrapper.getExchange();
                    //遮盖模式
                    if (page.getInfoArea() != null && page.getInfoArea().size() > 0) {
                        sliceConfig.addConfig(pageNumber, 0, 0, 0, 0);
                    }
                    //裁切模式
                    else if (page.getAnswerArea() != null && !page.getAnswerArea().isEmpty()) {
                        //获取卡格式图片原始尺寸
                        //此处图片base64编码特殊，需要采用Mime标准
                        Image image = ImageIO
                                .read(new ByteArrayInputStream(Base64.getMimeDecoder().decode(page.getPageImage())));
                        int width = image.getWidth(null);
                        int height = image.getHeight(null);
                        for (AnswerArea area : page.getAnswerArea()) {
                            sliceConfig.addConfig(pageNumber, (int) (area.getArea()[0] * width),
                                    (int) (area.getArea()[1] * height), (int) (area.getArea()[2] * width),
                                    (int) (area.getArea()[3] * height));
                        }
                    }
                }
            }
        }
        return sliceConfig;
    }

    public List<String> getSliceName() throws IOException {
        List<String> sliceName = new LinkedList<String>();
        if (pages != null) {
            for (CardPageWrapper pageWrapper : pages) {
                CardPage page = pageWrapper.getExchange();
                for (AnswerArea area : page.getAnswerArea()) {
                    if (area.getMainNumber() == null) {
                        throw new ParameterException("CET模式下裁切图必须指定大题号");
                    }
                    sliceName.add(area.getMainNumber().toString());
                }
            }
        }
        return sliceName;
    }

    /**
     * 输出到文件前，进行必要的字段值转换与处理
     */
    private void format() {
        version = "1.0.0";
        adapted = false;
        for (CardPageWrapper cpw : pages) {
            cpw.getExchange().format();
        }
    }

    /**
     * 输出为字节数组，用于保存到文件或其他输出流
     *
     * @return
     * @throws JsonProcessingException
     */
    public byte[] output() throws JsonProcessingException {
        ObjectMapper mapper = new ObjectMapper();
        this.format();
        return mapper.writeValueAsBytes(this);
    }

    /**
     * 从输入流解析卡格式文件
     *
     * @param content
     * @return
     * @throws IOException
     */
    public static CardFile parse(String content) throws IOException {
        ObjectMapper mapper = new ObjectMapper();
        mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        CardFile obj = mapper.readValue(content, CardFile.class);
//        obj.postInit();
        return obj;
    }

    private void postInit() {
        // 解析卡格式文件中的裁切图坐标，用于云阅卷同步
        this.sliceConfig = new SliceConfig();
        if (pages != null) {
            int pageNumber = 0;
            for (CardPageWrapper pageWrapper : pages) {
                pageNumber++;
                // 强制使用遮盖模式
                sliceConfig.addConfig(pageNumber, 0, 0, 0, 0);
                // 如果没有infoArea则补一个
                if (pageWrapper.getExchange().getInfoArea().isEmpty()) {
                    pageWrapper.getExchange().getInfoArea().add(new double[]{0, 0, 0.001, 0.001});
                }
            }
        }
        // 解析卡格式文件中的大题评卷坐标，用于云阅卷评卷分组设置
        this.markConfig = new MarkConfig();
        if (pages != null) {
            int pageNumber = 0;
            for (CardPageWrapper pageWrapper : pages) {
                pageNumber++;
                for (AnswerArea area : pageWrapper.getExchange().getAnswerArea()) {
                    if (area.getMainNumber() != null && area.getArea() != null && area.getArea().length >= 4) {
//                        markConfig.addConfig(area.getMainNumber(), pageNumber, area.getArea()[0], area.getArea()[1],
//                                area.getArea()[2], area.getArea()[3]);
                    }
                }
            }
        }
    }

    public static void main(String[] args) {
        String content = "{\"version\":\"2.0.0\",\"cardType\":\"STANDARD\",\"cardConfig\":{\"id\":\"437532692649615360\",\"createId\":\"437311641944596480\",\"createTime\":1695256722943,\"updateId\":null,\"updateTime\":1695256722943,\"schoolId\":\"1\",\"orgId\":\"437306348707782656\",\"name\":\"测试规则\",\"examNumberStyle\":\"PRINT\",\"examNumberDigit\":10,\"paperType\":\"PRINT\",\"examAbsent\":true,\"writeSign\":true,\"discipline\":true,\"requiredFields\":[{\"code\":\"studentName\",\"name\":\"姓名\",\"enable\":true,\"contentHtml\":\"<i>姓</i> <i>名</i>\"}],\"extendFields\":[{\"code\":\"studentCode\",\"name\":\"学号\",\"enable\":true,\"contentHtml\":\"<i>学</i> <i>号</i>\"},{\"code\":\"courseCode\",\"name\":\"课程代码\",\"enable\":true,\"contentHtml\":\"<i>课</i> <i>程</i> <i>代</i> <i>码</i>\"},{\"code\":\"courseName\",\"name\":\"课程名称\",\"enable\":true,\"contentHtml\":\"<i>课</i> <i>程</i> <i>名</i> <i>称</i>\"},{\"code\":\"paperNumber\",\"name\":\"试卷编号\",\"enable\":false},{\"code\":\"examPlace\",\"name\":\"校区\",\"enable\":false},{\"code\":\"examRoom\",\"name\":\"考试教室\",\"enable\":false},{\"code\":\"examDate\",\"name\":\"考试日期\",\"enable\":false},{\"code\":\"examTime\",\"name\":\"考试时间\",\"enable\":false},{\"code\":\"clazzName\",\"name\":\"班级\",\"enable\":false}],\"titleRule\":\"${schoolName}\",\"attention\":\"无\",\"objectiveAttention\":\"\",\"subjectiveAttention\":\"\",\"enable\":true,\"remark\":\"\",\"fillNumber\":10,\"orgIds\":null,\"pageSize\":\"A3\",\"columnNumber\":2,\"columnGap\":20,\"showForbidArea\":false,\"undertakingEnable\":false,\"undertakingBody\":\"在考试过程中，严格遵守考场纪律，服从监考老师安排，不请他（她）人代考，不将通讯设备、电子设备带入考场，不携带与有考试内容相关的纸质材料，不扰乱考场秩序，如违反，自愿承担相应责任。\",\"cardDesc\":\"\",\"makeMethod\":\"SELF\",\"aOrB\":false,\"cardTitle\":\"测试学校_1\"},\"pages\":[{\"type\":\"PAGE\",\"pageSize\":\"A3\",\"columnNumber\":2,\"columnGap\":20,\"locators\":{\"top\":[{\"id\":\"locator-0-00\",\"type\":\"LOCATOR\",\"x\":\"\",\"y\":\"\",\"w\":\"\",\"h\":\"\"},{\"id\":\"locator-0-01\",\"type\":\"LOCATOR\",\"x\":\"\",\"y\":\"\",\"w\":\"\",\"h\":\"\"}],\"bottom\":[{\"id\":\"locator-0-10\",\"type\":\"LOCATOR\",\"x\":\"\",\"y\":\"\",\"w\":\"\",\"h\":\"\"}]},\"globals\":[],\"columns\":[{\"type\":\"COLUMN\",\"x\":\"\",\"y\":\"\",\"w\":\"\",\"h\":\"\",\"isFull\":false,\"elements\":[{\"type\":\"CARD_HEAD\",\"x\":0,\"y\":0,\"w\":703,\"h\":384,\"cardTitle\":\"测试学校_1\",\"cardDesc\":\"\",\"aOrB\":false,\"paperType\":\"PRINT\",\"examAbsent\":true,\"writeSign\":true,\"fillNumber\":10,\"examNumberStyle\":\"PRINT\",\"businessParams\":[],\"attention\":\"无\",\"objectiveAttention\":\"\",\"subjectiveAttention\":\"\",\"columnNumber\":2,\"isSimple\":false,\"sign\":\"head\",\"id\":\"element-ljrd05coafdvllp8\",\"createId\":\"437311641944596480\",\"createTime\":1695256722943,\"updateId\":null,\"updateTime\":1695256722943,\"schoolId\":\"1\",\"orgId\":\"437306348707782656\",\"name\":\"测试规则\",\"examNumberDigit\":10,\"discipline\":true,\"requiredFields\":[{\"code\":\"studentName\",\"name\":\"姓名\",\"enable\":true,\"contentHtml\":\"<i>姓</i> <i>名</i>\"}],\"extendFields\":[{\"code\":\"studentCode\",\"name\":\"学号\",\"enable\":true,\"contentHtml\":\"<i>学</i> <i>号</i>\"},{\"code\":\"courseCode\",\"name\":\"课程代码\",\"enable\":true,\"contentHtml\":\"<i>课</i> <i>程</i> <i>代</i> <i>码</i>\"},{\"code\":\"courseName\",\"name\":\"课程名称\",\"enable\":true,\"contentHtml\":\"<i>课</i> <i>程</i> <i>名</i> <i>称</i>\"},{\"code\":\"paperNumber\",\"name\":\"试卷编号\",\"enable\":false},{\"code\":\"examPlace\",\"name\":\"校区\",\"enable\":false},{\"code\":\"examRoom\",\"name\":\"考试教室\",\"enable\":false},{\"code\":\"examDate\",\"name\":\"考试日期\",\"enable\":false},{\"code\":\"examTime\",\"name\":\"考试时间\",\"enable\":false},{\"code\":\"clazzName\",\"name\":\"班级\",\"enable\":false}],\"titleRule\":\"${schoolName}\",\"enable\":true,\"remark\":\"\",\"orgIds\":null,\"pageSize\":\"A3\",\"columnGap\":20,\"showForbidArea\":false,\"undertakingEnable\":false,\"undertakingBody\":\"在考试过程中，严格遵守考场纪律，服从监考老师安排，不请他（她）人代考，不将通讯设备、电子设备带入考场，不携带与有考试内容相关的纸质材料，不扰乱考场秩序，如违反，自愿承担相应责任。\",\"makeMethod\":\"SELF\",\"key\":\"c8k0rqdk3q1omjm2\",\"isCovered\":false},{\"type\":\"TOPIC_HEAD\",\"x\":0,\"y\":0,\"w\":0,\"h\":10,\"content\":\"\",\"typeName\":\"客观题\",\"isColumnFirst\":false,\"sign\":\"objective\",\"id\":\"element-f5e22pc89dphfaao\",\"key\":\"14ps0qlgh4rerp7o\"},{\"id\":\"element-h0teoq6gdped1fog\",\"topicNo\":1,\"topicName\":\"单选\",\"startNumber\":1,\"endNumber\":10,\"questionsCount\":10,\"optionCount\":4,\"optionDirection\":\"horizontal\",\"questionDirection\":\"vertical\",\"isBoolean\":false,\"booleanType\":\"√,×\",\"isMultiply\":false,\"key\":\"8s3s58e14usq4t14\",\"type\":\"FILL_QUESTION\",\"x\":0,\"y\":0,\"w\":703,\"h\":136,\"minHeight\":136,\"sign\":\"objective\",\"questionCountPerGroup\":5,\"groupPerLine\":5,\"questionGap\":8,\"groupGap\":27,\"optionGap\":6,\"isCovered\":false,\"fontSize\":\"14px\",\"parent\":{\"id\":\"element-8bm5crb82o6dbfmo\",\"topicNo\":1,\"topicName\":\"单选\",\"startNumber\":1,\"endNumber\":10,\"questionsCount\":10,\"optionCount\":4,\"optionDirection\":\"horizontal\",\"questionDirection\":\"vertical\",\"isBoolean\":false,\"booleanType\":\"√,×\",\"isMultiply\":false,\"key\":\"0qrptsnom66a67qg\",\"type\":\"FILL_QUESTION\",\"x\":0,\"y\":0,\"w\":703,\"h\":114,\"minHeight\":114,\"sign\":\"objective\",\"questionCountPerGroup\":5,\"groupPerLine\":4,\"questionGap\":8,\"groupGap\":30,\"optionGap\":6,\"isCovered\":false,\"fontSize\":\"14px\"},\"isLast\":true,\"elementSerialNo\":0},{\"id\":\"element-qj42k9b8qlnk57kj\",\"topicNo\":2,\"topicName\":\"多选\",\"startNumber\":1,\"endNumber\":10,\"questionsCount\":10,\"optionCount\":4,\"optionDirection\":\"horizontal\",\"questionDirection\":\"vertical\",\"isBoolean\":false,\"booleanType\":\"√,×\",\"isMultiply\":true,\"key\":\"msmp6ubd330dto8o\",\"type\":\"FILL_QUESTION\",\"x\":0,\"y\":0,\"w\":703,\"h\":136,\"minHeight\":136,\"sign\":\"objective\",\"questionCountPerGroup\":5,\"groupPerLine\":5,\"questionGap\":8,\"groupGap\":27,\"optionGap\":6,\"isCovered\":false,\"fontSize\":\"14px\",\"parent\":{\"id\":\"element-mmqqvdual16p2os8\",\"topicNo\":2,\"topicName\":\"多选\",\"startNumber\":1,\"endNumber\":10,\"questionsCount\":10,\"optionCount\":4,\"optionDirection\":\"horizontal\",\"questionDirection\":\"vertical\",\"isBoolean\":false,\"booleanType\":\"√,×\",\"isMultiply\":true,\"key\":\"6agnod7v8ljfflbl\",\"type\":\"FILL_QUESTION\",\"x\":0,\"y\":0,\"w\":703,\"h\":114,\"minHeight\":114,\"sign\":\"objective\",\"questionCountPerGroup\":5,\"groupPerLine\":4,\"questionGap\":8,\"groupGap\":30,\"optionGap\":6,\"isCovered\":false,\"fontSize\":\"14px\"},\"isLast\":true,\"elementSerialNo\":1},{\"id\":\"element-f1ope63g86frf0sg\",\"topicNo\":3,\"topicName\":\"凑数\",\"startNumber\":1,\"endNumber\":10,\"questionsCount\":10,\"optionCount\":2,\"optionDirection\":\"horizontal\",\"questionDirection\":\"vertical\",\"isBoolean\":true,\"booleanType\":\"√,×\",\"isMultiply\":false,\"key\":\"bqmtt2qoka4l6lhg\",\"type\":\"FILL_QUESTION\",\"x\":0,\"y\":0,\"w\":703,\"h\":136,\"minHeight\":136,\"sign\":\"objective\",\"questionCountPerGroup\":5,\"groupPerLine\":7,\"questionGap\":8,\"groupGap\":33,\"optionGap\":6,\"isCovered\":false,\"fontSize\":\"14px\",\"parent\":{\"id\":\"element-9n1bvsamovp8fthq\",\"topicNo\":3,\"topicName\":\"凑数\",\"startNumber\":1,\"endNumber\":10,\"questionsCount\":10,\"optionCount\":2,\"optionDirection\":\"horizontal\",\"questionDirection\":\"vertical\",\"isBoolean\":true,\"booleanType\":\"√,×\",\"isMultiply\":false,\"key\":\"2kckpovgo3iigbvg\",\"type\":\"FILL_QUESTION\",\"x\":0,\"y\":0,\"w\":703,\"h\":114,\"minHeight\":114,\"sign\":\"objective\",\"questionCountPerGroup\":5,\"groupPerLine\":4,\"questionGap\":8,\"groupGap\":30,\"optionGap\":6,\"isCovered\":false,\"fontSize\":\"14px\"},\"isLast\":true,\"elementSerialNo\":2}]},{\"type\":\"COLUMN\",\"x\":\"\",\"y\":\"\",\"w\":\"\",\"h\":\"\",\"isFull\":false,\"elements\":[{\"type\":\"TOPIC_HEAD\",\"x\":0,\"y\":0,\"w\":0,\"h\":0,\"content\":\"\",\"typeName\":\"主观题\",\"isColumnFirst\":true,\"sign\":\"subjective\",\"id\":\"element-jop2rhj8pkb53asg\",\"key\":\"an494bjr1jc4tb18\"},{\"type\":\"EXPLAIN\",\"x\":0,\"y\":0,\"w\":703,\"h\":458,\"minHeight\":60,\"sign\":\"subjective\",\"topicNo\":4,\"isCovered\":false,\"isLast\":true,\"isExtend\":false,\"showTitle\":true,\"serialNumber\":1,\"elements\":[],\"parent\":{\"id\":\"element-t0q49kq7nlnsarmg\",\"key\":\"3kejad28ske6fcst\",\"type\":\"EXPLAIN\",\"sign\":\"subjective\",\"topicNo\":4,\"topicName\":\"解答\",\"startNumber\":1,\"questionsCount\":1,\"w\":703,\"endNumber\":1},\"id\":\"element-ldjv8rm8uqu50nuh\",\"key\":\"luntrkcocr29g77o\",\"elementSerialNo\":3},{\"id\":\"element-no2ob6dch1mic4kg\",\"key\":\"hek5nvdgc3cth90g\",\"type\":\"COMPOSITION\",\"x\":0,\"y\":0,\"w\":703,\"h\":265,\"minHeight\":60,\"sign\":\"subjective\",\"topicNo\":5,\"isCovered\":false,\"isLast\":true,\"isExtend\":false,\"showTitle\":true,\"serialNumber\":0,\"elements\":[{\"id\":\"element-mtlf16dgpb906e0e\",\"key\":\"52e7v9cgo7vvuf5g\",\"type\":\"LINES\",\"x\":0,\"y\":0,\"w\":703,\"h\":215,\"sign\":\"\",\"lineCount\":5,\"lineSpacing\":40,\"margin\":0,\"bold\":\"1px\",\"color\":\"#000000\",\"style\":\"solid\",\"container\":{\"id\":\"element-no2ob6dch1mic4kg\",\"type\":\"COMPOSITION\"}}],\"parent\":{\"id\":\"element-eec2v3ak60dv9hpo\",\"key\":\"24gd15sofhjp9h48\",\"type\":\"COMPOSITION\",\"sign\":\"subjective\",\"topicNo\":5,\"topicName\":\"作文\",\"w\":703},\"elementSerialNo\":4}]}],\"exchange\":{\"card_type\":2,\"page_size\":\"A3\",\"page_image\":\"\",\"locator\":{\"top\":[[0.0504413619,0.0267379679,0.0151324086,0.0142602496],[0.8738965952,0.0267379679,0.0151324086,0.0142602496]],\"bottom\":[[0.0504413619,0.9590017825,0.0151324086,0.0142602496]]},\"fill_locator\":[],\"check_area\":{\"black_line\":[],\"white_line\":[]},\"barcode\":[{\"field\":\"examNumber\",\"area\":[0.0504413619,0.1310160428,0.2184741488,0.1176470588]}],\"qrcode\":[],\"ocr_area\":[],\"info_area\":[[0.0504413619,0.0534759358,0.4432534678,0.3422459893]],\"fill_area\":[{\"field\":\"absent\",\"index\":1,\"single\":true,\"horizontal\":true,\"items\":[{\"main_number\":null,\"sub_number\":null,\"options\":[[0.3394546028,0.3737188057,0.0151324086,0.0106951872]],\"recog_info\":[]}]},{\"field\":\"breach\",\"index\":1,\"single\":true,\"horizontal\":true,\"items\":[{\"main_number\":null,\"sub_number\":null,\"options\":[[0.4483764187,0.3737188057,0.0151324086,0.0106951872]],\"recog_info\":[]}]},{\"field\":\"question\",\"index\":1,\"single\":true,\"horizontal\":true,\"items\":[{\"main_number\":1,\"sub_number\":1,\"options\":[[0.0687263556,0.4438502674,0.0113493064,0.008912656],[0.0838587642,0.4438502674,0.0113493064,0.008912656],[0.0989911728,0.4438502674,0.0113493064,0.008912656],[0.1141235813,0.4438502674,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":1,\"sub_number\":2,\"options\":[[0.0687263556,0.4598930481,0.0113493064,0.008912656],[0.0838587642,0.4598930481,0.0113493064,0.008912656],[0.0989911728,0.4598930481,0.0113493064,0.008912656],[0.1141235813,0.4598930481,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":1,\"sub_number\":3,\"options\":[[0.0687263556,0.4759358289,0.0113493064,0.008912656],[0.0838587642,0.4759358289,0.0113493064,0.008912656],[0.0989911728,0.4759358289,0.0113493064,0.008912656],[0.1141235813,0.4759358289,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":1,\"sub_number\":4,\"options\":[[0.0687263556,0.4919786096,0.0113493064,0.008912656],[0.0838587642,0.4919786096,0.0113493064,0.008912656],[0.0989911728,0.4919786096,0.0113493064,0.008912656],[0.1141235813,0.4919786096,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":1,\"sub_number\":5,\"options\":[[0.0687263556,0.5080213904,0.0113493064,0.008912656],[0.0838587642,0.5080213904,0.0113493064,0.008912656],[0.0989911728,0.5080213904,0.0113493064,0.008912656],[0.1141235813,0.5080213904,0.0113493064,0.008912656]],\"recog_info\":[]}]},{\"field\":\"question\",\"index\":2,\"single\":true,\"horizontal\":true,\"items\":[{\"main_number\":1,\"sub_number\":6,\"options\":[[0.157629256,0.4438502674,0.0113493064,0.008912656],[0.1727616646,0.4438502674,0.0113493064,0.008912656],[0.1878940731,0.4438502674,0.0113493064,0.008912656],[0.2030264817,0.4438502674,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":1,\"sub_number\":7,\"options\":[[0.157629256,0.4598930481,0.0113493064,0.008912656],[0.1727616646,0.4598930481,0.0113493064,0.008912656],[0.1878940731,0.4598930481,0.0113493064,0.008912656],[0.2030264817,0.4598930481,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":1,\"sub_number\":8,\"options\":[[0.157629256,0.4759358289,0.0113493064,0.008912656],[0.1727616646,0.4759358289,0.0113493064,0.008912656],[0.1878940731,0.4759358289,0.0113493064,0.008912656],[0.2030264817,0.4759358289,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":1,\"sub_number\":9,\"options\":[[0.157629256,0.4919786096,0.0113493064,0.008912656],[0.1727616646,0.4919786096,0.0113493064,0.008912656],[0.1878940731,0.4919786096,0.0113493064,0.008912656],[0.2030264817,0.4919786096,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":1,\"sub_number\":10,\"options\":[[0.157629256,0.5080213904,0.0113493064,0.008912656],[0.1727616646,0.5080213904,0.0113493064,0.008912656],[0.1878940731,0.5080213904,0.0113493064,0.008912656],[0.2030264817,0.5080213904,0.0113493064,0.008912656]],\"recog_info\":[]}]},{\"field\":\"question\",\"index\":3,\"single\":false,\"horizontal\":true,\"items\":[{\"main_number\":2,\"sub_number\":1,\"options\":[[0.0687263556,0.5650623886,0.0113493064,0.008912656],[0.0838587642,0.5650623886,0.0113493064,0.008912656],[0.0989911728,0.5650623886,0.0113493064,0.008912656],[0.1141235813,0.5650623886,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":2,\"sub_number\":2,\"options\":[[0.0687263556,0.5811051693,0.0113493064,0.008912656],[0.0838587642,0.5811051693,0.0113493064,0.008912656],[0.0989911728,0.5811051693,0.0113493064,0.008912656],[0.1141235813,0.5811051693,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":2,\"sub_number\":3,\"options\":[[0.0687263556,0.5971479501,0.0113493064,0.008912656],[0.0838587642,0.5971479501,0.0113493064,0.008912656],[0.0989911728,0.5971479501,0.0113493064,0.008912656],[0.1141235813,0.5971479501,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":2,\"sub_number\":4,\"options\":[[0.0687263556,0.6131907308,0.0113493064,0.008912656],[0.0838587642,0.6131907308,0.0113493064,0.008912656],[0.0989911728,0.6131907308,0.0113493064,0.008912656],[0.1141235813,0.6131907308,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":2,\"sub_number\":5,\"options\":[[0.0687263556,0.6292335116,0.0113493064,0.008912656],[0.0838587642,0.6292335116,0.0113493064,0.008912656],[0.0989911728,0.6292335116,0.0113493064,0.008912656],[0.1141235813,0.6292335116,0.0113493064,0.008912656]],\"recog_info\":[]}]},{\"field\":\"question\",\"index\":4,\"single\":false,\"horizontal\":true,\"items\":[{\"main_number\":2,\"sub_number\":6,\"options\":[[0.157629256,0.5650623886,0.0113493064,0.008912656],[0.1727616646,0.5650623886,0.0113493064,0.008912656],[0.1878940731,0.5650623886,0.0113493064,0.008912656],[0.2030264817,0.5650623886,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":2,\"sub_number\":7,\"options\":[[0.157629256,0.5811051693,0.0113493064,0.008912656],[0.1727616646,0.5811051693,0.0113493064,0.008912656],[0.1878940731,0.5811051693,0.0113493064,0.008912656],[0.2030264817,0.5811051693,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":2,\"sub_number\":8,\"options\":[[0.157629256,0.5971479501,0.0113493064,0.008912656],[0.1727616646,0.5971479501,0.0113493064,0.008912656],[0.1878940731,0.5971479501,0.0113493064,0.008912656],[0.2030264817,0.5971479501,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":2,\"sub_number\":9,\"options\":[[0.157629256,0.6131907308,0.0113493064,0.008912656],[0.1727616646,0.6131907308,0.0113493064,0.008912656],[0.1878940731,0.6131907308,0.0113493064,0.008912656],[0.2030264817,0.6131907308,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":2,\"sub_number\":10,\"options\":[[0.157629256,0.6292335116,0.0113493064,0.008912656],[0.1727616646,0.6292335116,0.0113493064,0.008912656],[0.1878940731,0.6292335116,0.0113493064,0.008912656],[0.2030264817,0.6292335116,0.0113493064,0.008912656]],\"recog_info\":[]}]},{\"field\":\"question\",\"index\":5,\"single\":true,\"horizontal\":true,\"items\":[{\"main_number\":3,\"sub_number\":1,\"options\":[[0.0687263556,0.6862745098,0.0113493064,0.008912656],[0.0838587642,0.6862745098,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":3,\"sub_number\":2,\"options\":[[0.0687263556,0.7023172906,0.0113493064,0.008912656],[0.0838587642,0.7023172906,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":3,\"sub_number\":3,\"options\":[[0.0687263556,0.7183600713,0.0113493064,0.008912656],[0.0838587642,0.7183600713,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":3,\"sub_number\":4,\"options\":[[0.0687263556,0.734402852,0.0113493064,0.008912656],[0.0838587642,0.734402852,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":3,\"sub_number\":5,\"options\":[[0.0687263556,0.7504456328,0.0113493064,0.008912656],[0.0838587642,0.7504456328,0.0113493064,0.008912656]],\"recog_info\":[]}]},{\"field\":\"question\",\"index\":6,\"single\":true,\"horizontal\":true,\"items\":[{\"main_number\":3,\"sub_number\":6,\"options\":[[0.131147541,0.6862745098,0.0113493064,0.008912656],[0.1462799496,0.6862745098,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":3,\"sub_number\":7,\"options\":[[0.131147541,0.7023172906,0.0113493064,0.008912656],[0.1462799496,0.7023172906,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":3,\"sub_number\":8,\"options\":[[0.131147541,0.7183600713,0.0113493064,0.008912656],[0.1462799496,0.7183600713,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":3,\"sub_number\":9,\"options\":[[0.131147541,0.734402852,0.0113493064,0.008912656],[0.1462799496,0.734402852,0.0113493064,0.008912656]],\"recog_info\":[]},{\"main_number\":3,\"sub_number\":10,\"options\":[[0.131147541,0.7504456328,0.0113493064,0.008912656],[0.1462799496,0.7504456328,0.0113493064,0.008912656]],\"recog_info\":[]}]}],\"answer_area\":[{\"main_number\":4,\"sub_number\":1,\"area\":[0.5063051702,0.0534759358,0.4432534678,0.4081996435]},{\"main_number\":5,\"sub_number\":1,\"area\":[0.5063051702,0.4616755793,0.4432534678,0.2361853832]}],\"extension\":{\"barcode\":[],\"fill_area\":[],\"ocr_area\":[],\"qrcode\":[]}}},{\"type\":\"PAGE\",\"pageSize\":\"A3\",\"columnNumber\":2,\"columnGap\":20,\"locators\":{\"top\":[{\"id\":\"locator-1-00\",\"type\":\"LOCATOR\",\"x\":\"\",\"y\":\"\",\"w\":\"\",\"h\":\"\"},{\"id\":\"locator-1-01\",\"type\":\"LOCATOR\",\"x\":\"\",\"y\":\"\",\"w\":\"\",\"h\":\"\"}],\"bottom\":[{\"id\":\"locator-1-10\",\"type\":\"LOCATOR\",\"x\":\"\",\"y\":\"\",\"w\":\"\",\"h\":\"\"}]},\"globals\":[],\"columns\":[{\"type\":\"COLUMN\",\"x\":\"\",\"y\":\"\",\"w\":\"\",\"h\":\"\",\"isFull\":false,\"elements\":[]},{\"type\":\"COLUMN\",\"x\":\"\",\"y\":\"\",\"w\":\"\",\"h\":\"\",\"isFull\":false,\"elements\":[]}],\"exchange\":{\"card_type\":2,\"page_size\":\"A3\",\"page_image\":\"\",\"locator\":{\"top\":[[0.1109709962,0.0267379679,0.0151324086,0.0142602496],[0.9344262295,0.0267379679,0.0151324086,0.0142602496]],\"bottom\":[[0.9344262295,0.9590017825,0.0151324086,0.0142602496]]},\"fill_locator\":[],\"check_area\":{\"black_line\":[],\"white_line\":[]},\"barcode\":[],\"qrcode\":[],\"ocr_area\":[],\"info_area\":[],\"fill_area\":[],\"answer_area\":[],\"extension\":{\"barcode\":[],\"fill_area\":[],\"ocr_area\":[],\"qrcode\":[]}}}]}";
        try {
//            CardFile cardFile = CardFile.parse(content);
//            System.out.println(JSON.toJSONString(cardFile));
            String position = "[{\"h\":0.0958003565,\"i\":1,\"w\":0.4472534678,\"x\":0.0484413619,\"y\":0.8019215686},{\"h\":0.054802139,\"i\":1,\"w\":0.4472534678,\"x\":0.0484413619,\"y\":0.8937219251}]";
            List<PictureConfig> list = JSON.parseArray(position, PictureConfig.class);
            String markConfigAdd = getMarkConfigAdd(list, 0.015);
            System.out.println(11);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
}
